import java.io.*;
import java.util.Vector;
import com.nttdocomo.ui.*;

public abstract class IAppBase extends IApplication 
    implements SoftKeyListener, ComponentListener {

    // UIs
    Panel mainPanel;
    Panel resultPanel;
    Panel checkPanel;
    ListBox fileList;
    TextBox resultBox;
    TextBox checkBox;
    TextBox infoBox;

    // data
    /** fileSpects[][] = { { filename, encoding, counts, comment }, ... } */
    protected String fileSpecs[][];
    int index = 0;

    boolean virgin = true;

    long warmupTime;
    long averageTime;
    long averageTimeNoGC;
    long averageTimeOnMem;
    long averageTimeNoGCOnMem;

    BenchBase benchmark;

    protected abstract BenchBase createBenchmark();

    //// extends IApplication
    public void start() {
	if(benchmark == null) {
	    benchmark = createBenchmark();
	    try {
		loadFileSpecs();
	    } catch (Exception e) {
		throw new RuntimeException(e.toString());
	    }
	}
	mainPanel = new Panel();
	mainPanel.setTitle("select a file");
	mainPanel.setSoftLabel(Frame.SOFT_KEY_1, "Exit");
	mainPanel.setSoftLabel(Frame.SOFT_KEY_2, "Test");
	fileList = new ListBox(ListBox.SINGLE_SELECT, 4);
	for (int i=0; i<fileSpecs.length; i++) {
	    fileList.append(fileSpecs[i][0]);
	}
	infoBox = new TextBox("", 20, 4, TextBox.DISPLAY_ANY);
	mainPanel.add(fileList);
	mainPanel.add(infoBox);
	mainPanel.setComponentListener(this);
	mainPanel.setSoftKeyListener(this);

	resultPanel = new Panel();
	resultPanel.setTitle("test result:");
	resultPanel.setSoftLabel(Frame.SOFT_KEY_1, "Back");
	resultPanel.setSoftLabel(Frame.SOFT_KEY_2, "Check");
	resultBox = new TextBox("", 20, 10, TextBox.DISPLAY_ANY);
	resultPanel.add(resultBox);
	resultPanel.setSoftKeyListener(this);

	checkPanel = new Panel();
	checkPanel.setTitle("check result:");
	checkPanel.setSoftLabel(Frame.SOFT_KEY_1, "Back");
	checkBox = new TextBox("", 20, 10, TextBox.DISPLAY_ANY);
	checkPanel.add(checkBox);
	checkPanel.setSoftKeyListener(this);

	Display.setCurrent(mainPanel);
	fileList.select(0);
	fileList.requestFocus();
    }

    //// implements SoftKeyListener
    public void softKeyPressed(int key) {
	Frame f = Display.getCurrent();
	if (f == mainPanel) {
	    if (key == Frame.SOFT_KEY_1) {
		/// Exit
		terminate();
	    } else if(key == Frame.SOFT_KEY_2) {
		/// Test
		index = fileList.getSelectedIndex();
		resultBox.setText("wait...");
		Display.setCurrent(resultPanel);
		run(); //display.callSerially(this); 
	    }
	} else if (f == resultPanel) {
	    if (key == Frame.SOFT_KEY_1) {
		/// Back
		Display.setCurrent(mainPanel);
		fileList.requestFocus();
	    } else if(key == Frame.SOFT_KEY_2) {
		/// Check
		String correct = fileSpecs[index][2];
		String result = countToString();
		String out = "correct:\n  " + correct + "\n"
		    + "result:\n  " + result + "\n\n"
		    + (result.equals(correct) ? "OK." : "NG");
		checkBox.setText(out);
		Display.setCurrent(checkPanel);
	    }
	} else if (f == checkPanel) {
	    if (key == Frame.SOFT_KEY_1) {
		/// Back
		Display.setCurrent(resultPanel);
	    }
	}
    }

    public void softKeyReleased(int key) {}

    //// implements ComponentListener
    public void componentAction(Component source, int type, int param) {
	if (source == fileList && type == SELECTION_CHANGED) {
	    String out = fileSpecs[param][3] + "\n"
		+ "encoding:" + fileSpecs[param][1] + "\n"
		+ "cnt:" + fileSpecs[param][2];
	    infoBox.setText(out);
	}
    }

    ////////////////////////////////////////////////////////////////////////
    public void run() {
	String filename = fileSpecs[index][0];
	String encoding = fileSpecs[index][1];
	String out = "file:" + filename + "\n";
	try {
	    runBenchmark(filename, encoding);
	    out += "cnt:" + countToString() + "\n";
	    if (virgin) {
		out += "1st:" + warmupTime + "ms" + "\n";
		virgin = false;
	    }
	    //out += "avg:" + averageTime + "ms" + "\n";
	    out += "avg:" + averageTimeNoGC + "ms" + "\n";
	    //out += "avgM:" + averageTimeOnMem + "ms" + "\n";
	    //out += "ngcM:" + averageTimeNoGCOnMem + "ms";
	} catch (Exception ex) {
	    out = "ERROR:" + ex.toString();
	    ex.printStackTrace();
	}
	resultBox.setText(out);
	Display.setCurrent(resultPanel);
    }

    protected void runBenchmark(String filename, String encoding)
	throws Exception {
	benchmark.init(filename, encoding, false);
	warmupTime = benchmark.runWarmup();
	//averageTime = benchmark.runNormal(5);
	averageTimeNoGC = benchmark.runNoGC(5);

	//benchmark.init(filename, encoding, true);
	//averageTimeOnMem = benchmark.runNormal(5);
	//averageTimeNoGCOnMem = benchmark.runNoGC(5);
    }

    protected String countToString() {
	return benchmark.elementCount 
	    + "/" + benchmark.attributeCount 
	    + "/" + benchmark.characterCount;
    }

    protected void loadFileSpecs() throws Exception {
	InputStream s = getClass().getResourceAsStream("dir.txt");
	// line format of dir.txt:
	//   filename-1 encoding elms/atts/chars comment-in-english
	// filename must not have space character.
	//
	InputStreamReader reader = new InputStreamReader(s);
	boolean eol = false;
	Vector v = new Vector();
	StringBuffer buf = new StringBuffer();
	int c;
	int f = 0;
	String spec[] = new String[4];
	do {
	    c =  reader.read();
	    if (c == '\r' || c == '\n'|| c == -1) {
		if (eol) {
		    continue;
		} else {
		    eol = true;
		    spec[f] = buf.toString();
		    v.addElement(spec);
		    f = 0;
		    buf.setLength(0);
		    spec = new String[4];
		} 
	    } else {
		eol = false;
		if ((c == ' ' || c == '\t') && f < 3) {
		    spec[f] = buf.toString();
		    buf.setLength(0);
		    f++;
		} else {
		    buf.append((char)c);
		}
	    }
	} while(c != -1);
	fileSpecs = new String[v.size()][];
	for (int i=0; i<v.size(); i++) {
	    fileSpecs[i] = (String[])v.elementAt(i);
	    //System.err.println(fileSpecs[i][0]);
	}
    }

}
